/* Copyright (c) 2010 SpinCore Technologies, Inc.
 *   http://www.spincore.com
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */



/**
 * \file Frequency_Shift_Keying.c
 * \brief: This program demonstrates an implementation of Frequency Shift Keying (FSK)
 * \on a PulseBlasterDDS or RadioProcessor board. 
 * \ingroup mod
 */


#include <stdio.h>
#include "spinapi.h"

#define CLOCK_FREQ 75.0

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0

int main(int argc, char **argv)
{
	int start;
	int ret;
	int length;
	int i;
	int seq[1024];
	float fc0;
	float fc1;
	float tm;
	tm= 10;

	  printf ("Using spinapi library version %s\n", pb_get_version());
      if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());
        system("pause");
        return -1;
    }
    pb_set_clock(CLOCK_FREQ);
    pb_set_defaults();


	printf ("\n");
    printf
    ("This program demonstrates an implementation of \n"
     "Frequency Shift Keying (FSK) on a PulseBlasterDDS or RadioProcessor\n"
     "board. It will output an FSK waveform with a sine wave carrier.\n"
     "The user will input the corresponding\n"
     "frequencies for a logic 1 and a logic 0.\n"
     "The user will then input the length of the desired message and the\n"
     "contents of said message.\n"
     "The TTL output show the input message signal with a bit rate\n"
     "of 100 kbps. This is only the basics of what can be done with\n"
     "these boards. Please see their respective manuals and the source\n"
     "code of this program for more details.\n\n");

	printf ("Press CTRL-C at any time to quit\n\n");

    printf("Enter frequency of carrier for a logic 0 (in MHz): ");
    scanf("%f",&fc0);

    printf("Enter frequency of carrier for a logic 1 (in MHz): ");
    scanf("%f",&fc1);




	pb_start_programming(FREQ_REGS);
	pb_set_freq(fc0*MHz);
	pb_set_freq(fc1*MHz);
	pb_stop_programming();

	// Write 0.0 degrees to the first phase register of all channels
	pb_start_programming(TX_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();

	pb_start_programming(COS_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();

	pb_start_programming(SIN_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();

    //Taking the length of the logic sequence and accordingly storing the inputs in the array.

	printf("Please enter the length (in bits) of your message signal: ");
    scanf("%d",&length);
    for (i=0;i<length;i++)
    {
       printf("Please enter value of bit %d (1 or 0) : ",i+1);
       scanf("%d",&seq[i]);
    }

	pb_start_programming (PULSE_PROGRAM);

	// the next section sets the correct output on the TTL lines and for the carrier frequency
	// each bit is output for one bit period 

 for (i=0;i<length;i++)
       {
        if(!seq[i]) 
            {
                pb_inst_radio(0,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, tm*us);   //output the carrier wave for tm time period with frequency as assigned for the logic "0"
            }
        else  
            {
                pb_inst_radio(1,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x0F,CONTINUE, 0, tm*us);   //output the carrier wave for tm time period with frequency as assigned for the logic "1"

            }
        }



// Output no pulse for (tm*10) us. reset the phase. TTL outputs off. Execution
    // branches back to the beginning of the pulse program.



    ret =
    pb_inst_radio(0,0,0,0, TX_DISABLE, PHASE_RESET, NO_TRIGGER, 0x00,
                  BRANCH, 0, (tm*10)*us);

    if(ret < 0) {
        printf("Error with third instruction: %s\n", pb_get_error());
        return;
    }
	pb_stop_programming();

	// Trigger program
	pb_start();

	// Release control of the board
	pb_close();
	return 0;
}


